# Civilization 4 (c) 2007 Firaxis Games
# Dynamic Traits (c) 2010 Fredrik 'Grey Fox' Henriksson
# Website: http://greyfox.me/dynamictraits
# Contact: fredrik [dot] henriksson82 [at] gmail [dot] com
# CvDynamicTraitsEvents.py - Main Events Class

from CvPythonExtensions import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import sys
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser
import pickle
import CvEventManager

# globals
localText = CyTranslator()


# Grey Fox Speed Tweaks: START
import FoxDebug
import FoxTools 

FoxGlobals = {
	"USE_DEBUG_WINDOW" : True,
}

# END

#########################################################
class CvDynamicTraitsEvents(CvEventManager.CvEventManager):
	
	def __init__(self):
		
		self.bCtrl = False
		self.bShift = False
		self.bAlt = False
		self.bAllowCheats = False
		
		self.parent = CvEventManager.CvEventManager
		self.parent.__init__(self)
		self.mouse_IX = 0
		self.mouse_IY = 0
		self.last_mouse_IX = -1
		self.last_mouse_IY = -1
		
		self.Traits 		= {}
		self.Civics 		= {}
		self.Buildings 		= {}
		self.UnitCombats 	= {}
		self.Improvements 	= {}
		self.GreatPeople 	= {}
		
		self.Tools 			= None
		self.DbgWnd			= None
		
		
		self.Events={
			CvUtil.EventSetTurnsAutoPlayPopup : ('SetTurnsAutoPlayPopup', self.__eventSetTurnsAutoPlayApply, self.__eventSetTurnsAutoPlayBegin),
		}
	
	def __eventSetTurnsAutoPlayBegin(self, argslist):
		return 0

	def __eventSetTurnsAutoPlayApply(self, playerID, userData, popupReturn):
		if popupReturn.getButtonClicked() == 0: # ok button
			sEditBoxContent = popupReturn.getEditBoxString(0)
			if sEditBoxContent.isdigit():
				iNumTurns = int(sEditBoxContent)
				CyGame().setAIAutoPlay(iNumTurns)
		return 0

	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'

		eventType,key,mx,my,px,py = argsList
		game = CyGame()

		if (self.bAllowCheats):
			# notify debug tools of input to allow it to override the control
			argsList = (eventType,key,self.bCtrl,self.bShift,self.bAlt,mx,my,px,py,game.isNetworkMultiPlayer())
			if ( CvDebugTools.g_CvDebugTools.notifyInput(argsList) ):
				return 0

		if ( eventType == self.EventKeyDown ):
			theKey=int(key)

			CvCameraControls.g_CameraControls.handleInput( theKey )

			if (self.bAllowCheats):
				# Shift - T (Debug - No MP)
				if (theKey == int(InputTypes.KB_T)):
					if ( self.bShift ):
						self.beginEvent(CvUtil.EventAwardTechsAndGold)
						#self.beginEvent(CvUtil.EventCameraControlPopup)
						return 1

				elif (theKey == int(InputTypes.KB_W)):
					if ( self.bShift and self.bCtrl):
						self.beginEvent(CvUtil.EventShowWonder)
						return 1

				# Shift - ] (Debug - currently mouse-overd unit, health += 10
				elif (theKey == int(InputTypes.KB_LBRACKET) and self.bShift ):
					unit = map.plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = min( unit.maxHitPoints()-1, unit.getDamage() + 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )

				# Shift - [ (Debug - currently mouse-overd unit, health -= 10
				elif (theKey == int(InputTypes.KB_RBRACKET) and self.bShift ):
					unit = map.plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = max( 0, unit.getDamage() - 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )

				elif (theKey == int(InputTypes.KB_F1)):
					if ( self.bShift ):
						CvScreensInterface.replayScreen.showScreen(False)
						return 1
					# don't return 1 unless you want the input consumed

				elif (theKey == int(InputTypes.KB_F2)):
					if ( self.bShift ):
						import CvDebugInfoScreen
						CvScreensInterface.showDebugInfoScreen()
						return 1

				elif (theKey == int(InputTypes.KB_F3)):
					if ( self.bShift ):
						CvScreensInterface.showDanQuayleScreen(())
						return 1

				elif (theKey == int(InputTypes.KB_F4)):
					if ( self.bShift ):
						CvScreensInterface.showUnVictoryScreen(())
						return 1
			
			if (self.bAllowCheats):
				if (theKey == int(InputTypes.KB_Z) and self.bShift and self.bCtrl):
					self.Tools.showAutoPlayPopup()

			if ( self.bShift ):
				if (theKey == int(InputTypes.KB_C)):
					if FoxGlobals["USE_DEBUG_WINDOW"]:
						self.DbgWnd.redrawWindow(CyGame().getActivePlayer())

		return 0
	
	def loadData(self, bGameStart = False):
		' Loading data and initializing variables '
		self.Tools 			= FoxTools.FoxTools()
		self.Traits 		= self.Tools.getTraitDict()
		self.Civics 		= self.Tools.getCivicDict()
		self.Buildings 		= self.Tools.getBuildingDict()
		self.UnitCombats 	= self.Tools.getUnitCombatDict()
		self.Improvements 	= self.Tools.getImprovementDict()
		self.GreatPeople 	= self.Tools.getGreatPeopleDict()
		
		self.DbgWnd = FoxDebug.FoxDebug()
		if not bGameStart:
			self.DbgWnd.loadData(pickle.loads(CyGame().getScriptData()))
		
	def onLoadGame(self, argsList):
		# self.parent.onLoadGame(self, argsList)
		CvAdvisorUtils.resetNoLiberateCities()
		self.loadData()
		
		return 0
		
	def onPreSave(self, argsList):
		"called before a game is actually saved"
		# self.parent.onPreSave(self, argsList)
		if self.DbgWnd != None:
			CyGame().setScriptData(pickle.dumps(self.DbgWnd.saveData()))
	
# ######################################################
# 	Game Turns Events
# ######################################################		
	def onGameStart(self, argsList):
		'Called at the start of the game'
		# self.parent.onGameStart(self, argsList)
		self.loadData(True)
		
	def onBeginGameTurn(self, argsList):
		'Called at the beginning of the end of each turn'
		iGameTurn = argsList[0]
		# self.parent.onBeginGameTurn(self, argsList)
		
	def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]
		# self.parent.onEndGameTurn(self, argsList)
		
	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList
		# self.parent.onBeginPlayerTurn(self, argsList)
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iPlayer)
		getCounter 		= pPlayer.getTraitCounter
		changeCounter 	= pPlayer.changeTraitCounter
		
		Trait 			= self.Traits
		
		if FoxGlobals["USE_DEBUG_WINDOW"]:
			self.DbgWnd.updateTraitCounterTab(iPlayer, iGameTurn)
		
		if pPlayer.isHuman():
			self.Tools.showTraitPopup()
		
		
	def onEndPlayerTurn(self, argsList):
		'Called at the end of a players turn'
		iGameTurn, iPlayer = argsList
		# self.parent.onEndPlayerTurn(self, argsList)
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iPlayer)
		
		if pPlayer.isBarbarian(): return

		getCounter 		= pPlayer.getTraitCounter
		changeCounter 	= pPlayer.changeTraitCounter
		civilization 	= gc.getCivilizationInfo
		
		# Traits
		#if self.Traits == {}:
		#	self.Traits = getTraitDict()
		Trait 			= self.Traits
		iAgricultural	= Trait["Agricultural"]
		iAggressive 	= Trait["Aggressive"]
		iAnalytical 	= Trait["Analytical"]
		iAuthoritarian 	= Trait["Authoritarian"]
		iBenevolent		= Trait["Benevolent"]
		iCapitalist		= Trait["Capitalist"]
		iCommercial 	= Trait["Commercial"]
		iCruel 			= Trait["Cruel"]
		iDemocratic 	= Trait["Democratic"]
		iExplorers 		= Trait["Explorers"]
		iFeudalist 		= Trait["Feudalist"]
		iHorselords 	= Trait["Horselords"]
		iHumanitarian 	= Trait["Humanitarian"]
		iImperialist 	= Trait["Imperialist"]
		iIsolationist 	= Trait["Isolationist"]
		iLibertarian	= Trait["Libertarian"]
		iNationalist	= Trait["Nationalist"]
		iNonViolent		= Trait["Nonviolent"]
		iPeaceful 		= Trait["Peaceful"]
		iPhilosophical 	= Trait["Philosophical"]
		iProductive 	= Trait["Productive"]
		iScientific 	= Trait["Scientific"]
		iSocialist 		= Trait["Socialist"]
		iSpiritual	 	= Trait["Spiritual"]
		iTheocrat	 	= Trait["Theocrat"]
		
		# changeCounter(iDemocratic, 1000)
		# changeCounter(getInfoType('TRAIT_INDUSTRIOUS'), 1000)
		# changeCounter(getInfoType('TRAIT_PASTORALIST'), 1000)
			
		""" Traitcounters from Civics """
		Civic 			= self.Civics
		
		iGovernment = pPlayer.getCivics(Civic["Government"])
		
		if iGovernment == Civic["Hereditary Rule"]:
			iPercent = getPercent(getCounter(iAuthoritarian), 5, 4, 12)
			changeCounter(iAuthoritarian, iPercent)
			iPercent = getPercent(getCounter(iFeudalist), 8, 6, 16)
			changeCounter(iFeudalist, iPercent)

		elif iGovernment == Civic["Representation"]:
			iPercent = getPercent(getCounter(iDemocratic), 8, 6, 16)
			changeCounter(iDemocratic, iPercent)

		elif iGovernment == Civic["Police State"]:
			iPercent = getPercent(getCounter(iAuthoritarian), 12, 12, 64)
			changeCounter(iAuthoritarian, iPercent)
			iPercent = getPercent(getCounter(iNationalist), 8, 6, 18)
			changeCounter(iNationalist, iPercent)
			if getCounter(iDemocratic) > 0:
				iPercent = getPercent(getCounter(iDemocratic), 6, 2, 12)
				changeCounter(iDemocratic, -iPercent)

		elif iGovernment == Civic["Universal Suffrage"]:
			iPercent = getPercent(getCounter(iHumanitarian), 8, 6, 16)
			changeCounter(iHumanitarian, iPercent)
			iPercent = getPercent(getCounter(iDemocratic), 12, 12, 25)
			changeCounter(iDemocratic, iPercent)
		
		if iGovernment != Civic["Police State"] and iGovernment != Civic["Hereditary Rule"]:
			if getCounter(iAuthoritarian) > 0:
				iPercent = getPercent(getCounter(iAuthoritarian), 10, 5)
				changeCounter(iAuthoritarian, -iPercent)

		
		iLegal = pPlayer.getCivics(Civic["Legal"])
		
		if iLegal == Civic["Vassalage"]:
			iPercent = getPercent(getCounter(iFeudalist), 4, 3, 16)
			changeCounter(iFeudalist, iPercent)
		
		elif iLegal == Civic["Bureaucracy"]:
			iPercent = getPercent(getCounter(iProductive), 5, 2)
			changeCounter(iProductive, iPercent)
		
		elif iLegal == Civic["Nationhood"]:
			iPercent = getPercent(getCounter(iAnalytical), 5, 4)
			changeCounter(iAnalytical, iPercent)
			iPercent = getPercent(getCounter(iNationalist), 6, 4, 25)
			changeCounter(iNationalist, iPercent)
		
		elif iLegal == Civic["Free Speech"]:
			iPercent = getPercent(getCounter(iHumanitarian), 5, 4, 12)
			changeCounter(iHumanitarian, iPercent)
			iPercent = getPercent(getCounter(iDemocratic), 8, 6, 18)
			changeCounter(iDemocratic, iPercent)
			iPercent = getPercent(getCounter(iNationalist), 4, 2, 4)
			changeCounter(iNationalist, -iPercent)

		iLabor = pPlayer.getCivics(Civic["Labor"])

		if iLabor == Civic["Slavery"]:
			iPercent = getPercent(getCounter(iCruel), 4, 2, 6)
			changeCounter(iCruel, iPercent)
			iCounter = getCounter( iBenevolent )
			if iCounter > 0:
				iPercent = getPercent( iCounter, 4, 2, 8 )  
				changeCounter( iBenevolent, -iPercent )

		elif iLabor == Civic["Serfdom"]:
			iPercent = getPercent(getCounter(iFeudalist), 3, 4, 8)
			changeCounter(iFeudalist, iPercent)

		elif iLabor == Civic["Caste System"]:
			iPercent = getPercent(getCounter(iPhilosophical), 3, 3, 5)
			changeCounter(iPhilosophical, iPercent)

		elif iLabor == Civic["Emancipation"]:
			iPercent = getPercent(getCounter(iHumanitarian), 2, 3)
			changeCounter(iHumanitarian, iPercent)
			iPercent = getPercent(getCounter(iDemocratic), 8, 8, 32)
			changeCounter(iDemocratic, iPercent)
			if getCounter(iCruel) > 0:
				iPercent = getPercent(getCounter(iCruel), 5, 3, 3)
				changeCounter(iCruel, -iPercent)
			
		if iLabor != Civic["Slavery"]:
			if getCounter(iCruel) > 0:
				iPercent = getPercent(getCounter(iCruel), 10, 5)
				changeCounter(iCruel, -iPercent)
		
		iEconomy = pPlayer.getCivics(Civic["Economy"])

		if iEconomy == Civic["Mercentalism"]:
			iPercent = getPercent(getCounter(iIsolationist), 8, 8, 32)
			changeCounter(iIsolationist, iPercent)

		elif iEconomy == Civic["Free Market"]:
			iPercent = getPercent(getCounter(iCommercial), 5, 6, 32)
			changeCounter(iCommercial, iPercent)
			iPercent = getPercent(getCounter(iCapitalist), 12, 12, 48)
			changeCounter(iCapitalist, iPercent)
			iPercent = getPercent(getCounter(iLibertarian), 4, 4, 10)
			changeCounter(iLibertarian, iPercent)

		elif iEconomy == Civic["State Property"]:
			iPercent = getPercent(getCounter(iSocialist), 5, 1, 25)
			changeCounter(iSocialist, iPercent)
		
		elif iEconomy == Civic["Environmentalism"]:
			iPercent = getPercent(getCounter(iHumanitarian), 10, 8, 32)
			changeCounter(iHumanitarian, iPercent)
			
		if iEconomy != Civic["State Property"]:
			if getCounter(iSocialist) > 0:
				iPercent = getPercent(getCounter(iSocialist), 12, 12, 48)
				changeCounter(iSocialist, -iPercent)

		iReligion = pPlayer.getCivics(Civic["Religion"])

		if iReligion == Civic["Organized Religion"]:
			iPercent = getPercent(getCounter(iSpiritual), 4, 2, 5)
			changeCounter(iSpiritual, iPercent)
			
		elif iReligion == Civic["Theocracy"]:
			iPercent = getPercent(getCounter(iAuthoritarian), 4, 2, 10)
			changeCounter(iAuthoritarian, iPercent)
			iPercent = getPercent(getCounter(iTheocrat), 6, 3, 12)
			changeCounter(iTheocrat, iPercent)
			if getCounter(iDemocratic) > 0:
				iPercent = getPercent(getCounter(iDemocratic), 4, 1, 6)
				changeCounter(iDemocratic, -iPercent)
			
		elif iReligion == Civic["Pacifism"]:
			iPercent = getPercent(getCounter(iPeaceful), 2, 1, 3)
			changeCounter(iPeaceful, iPercent)
			iPercent = getPercent(getCounter(iNonViolent), 2, 1, 3)
			changeCounter(iNonViolent, iPercent)
			iPercent = getPercent(getCounter(iBenevolent), 2, 1, 3)
			changeCounter(iBenevolent, iPercent)

		elif iReligion == Civic["Free Religion"]:
			iPercent = getPercent(getCounter(iScientific), 5, 3, 5)
			changeCounter(iScientific, iPercent)
			iPercent = getPercent(getCounter(iLibertarian), 5, 3, 5)
			changeCounter(iLibertarian, iPercent)
			iPercent = getPercent(getCounter(iSpiritual), 8, 8, 32)
			changeCounter(iSpiritual, -iPercent)
				
		if iReligion != Civic["Theocracy"]:
			if getCounter(iAuthoritarian) > 0:
				iPercent = getPercent(getCounter(iAuthoritarian), 10, 4, 12)
				changeCounter(iAuthoritarian, -iPercent)
			if getCounter(iTheocrat) > 0:
				iPercent = getPercent(getCounter(iTheocrat), 10, 4, 12)
				changeCounter(iTheocrat, -iPercent)

		iPercent = getPercent(getCounter(iAggressive), 2, 1)
		changeCounter(iAggressive, -iPercent)
		
		pTeam = gc.getTeam(pPlayer.getTeam())
		bValidPeaceful = False
		if pTeam.getHasMetCivCount(True) > 0:
			if pTeam.getAtWarCount(True) == 0:
				bValidPeaceful = True

		if iGameTurn % 2 == 1:
			iPercent = getPercent( getCounter( iBenevolent ), 5, 2, 8 )  
			changeCounter( iBenevolent, iPercent )
			if bValidPeaceful:
				iPercent = getPercent( getCounter( iPeaceful ), 5, 2, 8)  
				changeCounter( iPeaceful, iPercent )
			if getCounter(iCruel) > 0:
				iPercent = getPercent(getCounter(iCruel), 10, 4)
				changeCounter(iCruel, -iPercent)
		elif iGameTurn % 4 == 1:
			changeCounter(iHorselords, -1)
			iPercent = getPercent(getCounter(iExplorers), 2, 1, 5)
			changeCounter(iExplorers, -iPercent)
		elif iGameTurn % 6 == 1:		
			if getCounter(iAgricultural) > 0:
				changeCounter(iAgricultural, -1)
			if getCounter(iProductive) > 0:
				changeCounter(iProductive, -1)
					
		try:
			scriptData = (pickle.loads(pPlayer.getScriptData()))
		except:
			return
			
		iMaxPlayer = gc.getMAX_PLAYERS()
		bValid = False
		for iLoopPlayer in xrange(iMaxPlayer):
			pLoopPlayer = getPlayer(iLoopPlayer)
			if pLoopPlayer.isAlive():
				szName = gc.getCivilizationInfo(pLoopPlayer.getCivilizationType()).getDescription()
				if scriptData[szName] > 0:
					iTemp = scriptData[szName] 
					scriptData[szName] = (iTemp - 1)
					bValid = True
		if bValid:
			pPlayer.setScriptData(pickle.dumps(scriptData))
		

# ######################################################
# 	Player / Civilization / Team Events
# ######################################################		
	def createPlayerData(self, pPlayer):
		' Creates the initial script data of a player '
		gc = CyGlobalContext() 
		getPlayer = gc.getPlayer
		religion = gc.getReligionInfo
		civilization = gc.getCivilizationInfo
		
		scriptData = {}
		for iRel in xrange(gc.getNumReligionInfos()):
			if iRel != -1:
				szName = religion(iRel).getDescription()
				scriptData[szName] = False
		
		iMaxPlayer 	= gc.getMAX_PLAYERS()
		for iLoopPlayer in xrange(iMaxPlayer):
			pLoopPlayer = getPlayer(iLoopPlayer)
			if pLoopPlayer.isAlive():
				szName = civilization(pLoopPlayer.getCivilizationType()).getDescription()
				scriptData[szName] = False
		
		pPlayer.setScriptData(pickle.dumps(scriptData))
		
	def onChangeWar(self, argsList):
		'War Status Changes'
		# self.parent.onChangeWar(self, argsList)
		bIsWar 			= argsList[0]
		iTeam 			= argsList[1]
		iRivalTeam 		= argsList[2]
		
		if self.Tools == None: return
		""" OK LETS SEE IF THIS WORK """
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		Trait 			= self.Traits
		iAggressive		= Trait["Aggressive"]
		iImperialist	= Trait["Imperialist"]
		iPeaceful		= Trait["Peaceful"]
		iNonViolent		= Trait["Nonviolent"]
		iBenevolent		= Trait["Benevolent"]
		
		if bIsWar:
			for iPlayer in xrange(gc.getMAX_PLAYERS()):
				pLoopPlayer = getPlayer(iPlayer)
				if pLoopPlayer.isAlive() and iTeam == pLoopPlayer.getTeam():
					getCounter 		= pLoopPlayer.getTraitCounter
					changeCounter 	= pLoopPlayer.changeTraitCounter
					iPercent 		= getPercent(getCounter(iAggressive), 10, 10, 25)
					changeCounter(iAggressive, iPercent)
					iPercent 		= getPercent(getCounter(iImperialist), 5, 5, 10)
					changeCounter(iImperialist, iPercent)
					iPercent 		= getPercent(getCounter(iPeaceful), 10, 18, 54)
					changeCounter(iPeaceful, -iPercent)
					iPercent 		= getPercent(getCounter(iNonViolent), 10, 16, 48)
					changeCounter(iNonViolent, -iPercent)
					iPercent		= getPercent(getCounter(iBenevolent), 10, 10, 32)
					changeCounter(iBenevolent, -iPercent)
		
		
	def onVassalState(self, argsList):
		'Vassal State'
		# self.parent.onVassalState(self, argsList)
		iMaster, iVassal, bVassal = argsList
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		Trait 			= self.Traits
		iFeudalist		= Trait["Feudalist"]
		
		for iPlayer in xrange(gc.getMAX_PLAYERS()):
			pLoopPlayer = getPlayer(iPlayer)
			if pLoopPlayer.isAlive():
				# Towards Feudalist if Master
				if iMaster == pLoopPlayer.getTeam():
					iPercent = getPercent(pLoopPlayer.getTraitCounter(iFeudalist), 100, 50, 100)
					pLoopPlayer.changeTraitCounter(iFeudalist, iPercent)
				# Vassal gets some points too since they are part of the system
				elif iVassal == pLoopPlayer.getTeam():
					iPercent = getPercent(pLoopPlayer.getTraitCounter(iFeudalist), 25, 12, 25)
					pLoopPlayer.changeTraitCounter(iFeudalist, iPercent)
			

	def onCityBuilt(self, argsList):
		'City Built'
		# self.parent.onCityBuilt(self, argsList)
		pCity 			= argsList[0]
		iOwner			= pCity.getOwner()
		
		gc 				= CyGlobalContext() 
		getInfoType		= gc.getInfoTypeForString
		pPlayer 		= gc.getPlayer(iOwner)
		initUnit 		= pPlayer.initUnit
		iSouth		= DirectionTypes.DIRECTION_SOUTH
		iNoAI		= UnitAITypes.NO_UNITAI
		
		iX = pCity.getX()
		iY = pCity.getY()
		
		if self.Tools.isBarbarian(iOwner): return
		
		pPlot 					= pCity.plot()
		gc 						= CyGlobalContext() 
		getPercent 			= self.Tools.getPercent
		self.createCityData(pCity)
		pPlayer 				= gc.getPlayer(iOwner)
		getCounter 			= pPlayer.getTraitCounter
		changeCounter 	= pPlayer.changeTraitCounter
		Trait 					= self.Traits
		
		iNumCities = pPlayer.getNumCities()
		if iNumCities > 1:
			# Expansive
			randNum = CyGame().getSorenRandNum
			iPercent = getPercent(getCounter(Trait["Expansive"]), 2, 4, 25)
			changeCounter(Trait["Expansive"], iPercent)
			
			# Imperialist
			iRnd = ((randNum(((iNumCities+1)*4), "Imperialist Random")))
			if iRnd <= 1: iRnd = 4;
			iPercent = getPercent(getCounter(Trait["Imperialist"]), iRnd, 12, 48)
			changeCounter(Trait["Imperialist"], iRnd)
			
			# Fishermen
			if pCity.isCoastal(gc.getDefineINT("MIN_WATER_SIZE_FOR_OCEAN")):
				iPercent = getPercent(getCounter(Trait["Fishermen"]), 4, 12, 48)
				changeCounter(Trait["Fishermen"], iRnd)
				

	def onCityRazed(self, argsList):
		'City Razed'
		# self.parent.onCityRazed(self, argsList)
		pCity, iPlayer = argsList
		iOwner 			= pCity.findHighestCulture()
		
		if self.Tools.isBarbarian(iOwner): return
		
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent 
		pPlayer 		= gc.getPlayer(iPlayer)
		Trait 			= self.Traits
		iAggressive 	= Trait["Aggressive"]
		iRaiders 		= Trait["Raiders"]
		
		randNum			= CyGame().getSorenRandNum
		getCounter 		= pPlayer.getTraitCounter
		changeCounter 	= pPlayer.changeTraitCounter
		iNumCities = 1
		pOwner = gc.getPlayer(iOwner)
		if pOwner:
			iNumCities = pOwner.getNumCities()
		iRnd = ((randNum((iNumCities*6), "Aggressive Random")) / 2)
		if iRnd < 5: iRnd = 5;
		
		iPercent = getPercent(getCounter(iAggressive), iRnd, 24, 128)
		changeCounter(iAggressive, iPercent)
		
		iPercent = getPercent(getCounter(iRaiders), (iRnd/2), 12, 128)
		changeCounter(iRaiders, iPercent)
		
	def onCityAcquired(self, argsList):
		'City Acquired'
		# self.parent.onCityAcquired(self, argsList)
		iPreviousOwner,iNewOwner,pCity,bConquest,bTrade = argsList
			
		if self.Tools.isBarbarian(iNewOwner): return
		
		gc 				= CyGlobalContext() 
		pPlayer 		= gc.getPlayer(iNewOwner)
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iNewOwner)
		pPrevious 		= getPlayer(iPreviousOwner)
		
		Trait 			= self.Traits
		iAggressive 	= Trait["Aggressive"]
		iImperialist 	= Trait["Imperialist"]
		iDiplomatic		= Trait["Diplomatic"]
		iTraders	 	= Trait["Traders"]
		
		if bConquest:
			randNum = CyGame().getSorenRandNum
			iRnd = ((randNum((pPrevious.getNumCities()*5), "Aggressive Random")) / 4)
			if iRnd <= 1: iRnd = 2;
			iPercent = getPercent(pPlayer.getTraitCounter(iAggressive), iRnd, iRnd)
			pPlayer.changeTraitCounter(iAggressive, iPercent)
			
			iRnd = ((randNum((pPlayer.getNumCities()*2), "Imperialist Random")))
			if iRnd <= 1: iRnd = 2;
			if iRnd > 15: iRnd = 15;
			iPercent = getPercent(pPlayer.getTraitCounter(iImperialist), iRnd, iRnd)
			pPlayer.changeTraitCounter(iImperialist, iPercent)
			
		if bTrade:
			iPercent = getPercent(pPlayer.getTraitCounter(iTraders), 25, 16)
			pPlayer.changeTraitCounter(iTraders, iPercent)
			
			iPercent = getPercent(pPlayer.getTraitCounter(iDiplomatic), 25, 16)
			pPlayer.changeTraitCounter(iDiplomatic, iPercent)
			
			iPercent = getPercent(pPrevious.getTraitCounter(iTraders), 10, 8)
			pPrevious.changeTraitCounter(iTraders, iPercent)
			
			iPercent = getPercent(pPrevious.getTraitCounter(iDiplomatic), 10, 8)
			pPrevious.changeTraitCounter(iDiplomatic, iPercent)
	
	def onCityHurry(self, argsList):
		'City is renamed'
		# self.parent.onCityHurry(self, argsList)
		pCity 			= argsList[0]
		iHurryType 		= argsList[1]
		iOwner 			= pCity.getOwner()
		
		if self.Tools.isBarbarian(iOwner): return
		
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		pPlayer			= getPlayer(iOwner)
		
		Trait 			= self.Traits
		
		iCruel 			= Trait["Cruel"]
		iCommercial 	= Trait["Commercial"]
		
		# Pop Rushing via Slavery
		if iHurryType == 0:
			iPercent = getPercent(pPlayer.getTraitCounter(iCruel), 5, 6, 24)
			pPlayer.changeTraitCounter(iCruel, iPercent)
		
		# Gold Rushing
		if iHurryType == 1:
			iPercent = getPercent(pPlayer.getTraitCounter(iCommercial), 8, 6, 24)
			pPlayer.changeTraitCounter(iCommercial, iPercent)
	
	def onPlayerChangeStateReligion(self, argsList):
		'Player changes his state religion'
		# self.parent.onPlayerChangeStateReligion(self, argsList)
		iPlayer, iNewReligion, iOldReligion = argsList
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		pPlayer 		= gc.getPlayer(iPlayer)
		religion 		= gc.getReligionInfo
		
		try:
			scriptData = (pickle.loads(pPlayer.getScriptData()))
		except:
			self.createPlayerData(pPlayer)
			scriptData = (pickle.loads(pPlayer.getScriptData()))
			
		szName = religion(iNewReligion).getDescription()
		if not scriptData[szName]:
			pPlayer.changeTraitCounter(gc.getInfoTypeForString('TRAIT_SPIRITUAL'), 5)
			scriptData[szName] = True
			pPlayer.setScriptData(pickle.dumps(scriptData))
	
	def onPlayerGoldTrade(self, argsList):
		'Player Trades gold to another player'
		# self.parent.onPlayerGoldTrade(self, argsList)
		iFromPlayer, iToPlayer, iGoldAmount = argsList
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		game		 	= CyGame()
		iGameTurn 		= game.getGameTurn()
		
		# Need to rewrite this so it works
		"""
		iGold = iGoldAmount * gc.getGameSpeedInfo(game.getGameSpeedType()).getGrowthPercent()
		iGold /= 100
		iGold -= (((iGameTurn*10)/25) + 25)
		if iGold <= 0:
			return
		
		pFromPlayer = getPlayer(iFromPlayer)
		pToPlayer = getPlayer(iToPlayer)
		szNameToPlayer = ""
		try:
			scriptDataFromPlayer = (pickle.loads(pFromPlayer.getScriptData()))
		except:
			self.createPlayerData(pFromPlayer)
			scriptDataToPlayer = (pickle.loads(pFromPlayer.getScriptData()))
		szNameToPlayer = gc.getCivilizationInfo(pToPlayer.getCivilizationType()).getDescription()
		
		try:
			scriptDataToPlayer = (pickle.loads(pToPlayer.getScriptData()))
		except:
			self.createPlayerData(pToPlayer)
			scriptDataToPlayer = (pickle.loads(pToPlayer.getScriptData()))
		szNameFromPlayer = gc.getCivilizationInfo(pFromPlayer.getCivilizationType()).getDescription()
		
		if scriptDataFromPlayer[szNameToPlayer] > 0 and scriptDataToPlayer[szNameFromPlayer] > 0:
			return
	
		Trait 			= self.Traits
		iTraders		= Trait["Traders"]
		
		if scriptDataFromPlayer[szNameToPlayer] == 0:
			pFromPlayer.changeTraitCounter(iTraders, 3)
			scriptDataFromPlayer[szNameToPlayer] = 10
			pFromPlayer.setScriptData(pickle.dumps(scriptDataFromPlayer))
		if scriptDataToPlayer[szNameFromPlayer] == 0:
			pToPlayer.changeTraitCounter(iTraders, 2)
			scriptDataToPlayer[szNameFromPlayer] = 10
			pToPlayer.setScriptData(pickle.dumps(scriptDataToPlayer))
		"""
		
	def onRouteBuilt(self, argsList):
		'Route Built'
		# self.parent.onRouteBuilt(self, argsList)
		iRoute, iX, iY = argsList
		
		pPlot = CyMap().plot(iX, iY)
		if pPlot.isOwned():
			gc 				= CyGlobalContext() 
			Trait 			= self.Traits
			iProductive 	= Trait["Productive"]
			iOrganized 		= Trait["Organized"]
			iOwner 			= pPlot.getOwner()
			
			pPlayer 		= gc.getPlayer(iOwner)
			getCounter 		= pPlayer.getTraitCounter
			changeCounter 	= pPlayer.changeTraitCounter
			getPercent 		= self.Tools.getPercent
			changeCounter(iProductive, getPercent(getCounter(iProductive), 4, 2, 4))
			changeCounter(iOrganized, getPercent(getCounter(iOrganized), 4, 2, 4))
	
	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		# self.parent.onImprovementBuilt(self, argsList)
		iImprovement, iX, iY = argsList
		
		if self.Tools == None: return
		
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		pPlot 			= CyMap().plot(iX, iY)
		getPlayer 		= gc.getPlayer
		iImprovement 	= gc.getImprovementInfo(iImprovement)
		
		try:
			scriptData = (pickle.loads(pPlot.getScriptData()))
		except:
			scriptData = {
				"hasHadImprovement" : False,
			}	
		
		"""
			WARNING:
			If this is a mod where forts can be made outside of culture, 
			then this might cause an error
		"""
		
		bValid = False
		if not scriptData["hasHadImprovement"]:
			if pPlot.isOwned():
				iOwner = pPlot.getOwner()
				
				if self.Tools.isBarbarian(iOwner): return
				
				Improvement 	= self.Improvements
				Trait 			= self.Traits
				pPlayer 		= getPlayer(iOwner)
				getCounter 		= pPlayer.getTraitCounter
				changeCounter 	= pPlayer.changeTraitCounter
				changeCounter(Trait["Productive"], getPercent(getCounter(Trait["Productive"]), 4, 2, 4))
				if iImprovement == Improvement["Farm"]:
					changeCounter(Trait["Agricultural"], getPercent(getCounter(Trait["Agricultural"]), 6, 8, 24))
					changeCounter(Trait["Agrarianist"], getPercent(getCounter(Trait["Agrarianist"]), 6, 6, 18))
					bValid = True
				elif iImprovement == Improvement["Fishing Boats"]:
					changeCounter(Trait["Fishermen"], getPercent(getCounter(Trait["Fishermen"]), 25, 24, 64))
					changeCounter(Trait["Seafaring"], getPercent(getCounter(Trait["Seafaring"]), 6, 8, 24))
					bValid = True
				elif iImprovement == Improvement["Whaling Boats"]:
					changeCounter(Trait["Fishermen"], getPercent(getCounter(Trait["Fishermen"]), 34, 36, 96))
					changeCounter(Trait["Seafaring"], getPercent(getCounter(Trait["Seafaring"]), 8, 12, 36))
					bValid = True
				elif iImprovement == Improvement["Mine"]:
					changeCounter(Trait["Productive"], getPercent(getCounter(Trait["Productive"]), 8, 8, 24))
					bValid = True
				elif iImprovement == Improvement["Workshop"]:
					changeCounter(Trait["Productive"], getPercent(getCounter(Trait["Productive"]), 12, 12, 32))
					bValid = True
				elif iImprovement == Improvement["Lumbermill"]:
					changeCounter(Trait["Productive"], getPercent(getCounter(Trait["Productive"]), 12, 12, 32))
					bValid = True
				elif iImprovement == Improvement["Windmill"]:
					changeCounter(Trait["Agricultural"], getPercent(getCounter(Trait["Agricultural"]), 12, 12, 32))
					bValid = True
				elif iImprovement == Improvement["Watermill"]:
					changeCounter(Trait["Productive"], getPercent(getCounter(Trait["Productive"]), 12, 12, 36))
					bValid = True
				elif iImprovement == Improvement["Plantation"]:
					changeCounter(Trait["Financial"], getPercent(getCounter(Trait["Financial"]), 8, 8, 24))
					changeCounter(Trait["Commercial"], getPercent(getCounter(Trait["Commercial"]), 8, 8, 24))
					bValid = True
				elif iImprovement == Improvement["Quarry"]:
					changeCounter(Trait["Productive"], getPercent(getCounter(Trait["Productive"]), 25, 32, 64))
					bValid = True
				elif iImprovement == Improvement["Pasture"]:
					changeCounter(Trait["Agricultural"], getPercent(getCounter(Trait["Agricultural"]), 8, 8, 24))
					changeCounter(Trait["Pastoralist"], getPercent(getCounter(Trait["Pastoralist"]), 32, 32, 96))
					bValid = True
				elif iImprovement == Improvement["Camp"]:
					changeCounter(Trait["Commercial"], getPercent(getCounter(Trait["Commercial"]), 12, 18, 48))
					changeCounter(Trait["Traders"], getPercent(getCounter(Trait["Traders"]), 12, 16, 32))
					bValid = True
				elif iImprovement == Improvement["Well"]:
					changeCounter(Trait["Commercial"], getPercent(getCounter(Trait["Commercial"]), 24, 32, 96))
					changeCounter(Trait["Capitalist"], getPercent(getCounter(Trait["Capitalist"]), 24, 32, 96))
					bValid = True
				elif iImprovement == Improvement["Offshore Platform"]:
					changeCounter(Trait["Commercial"], getPercent(getCounter(Trait["Commercial"]), 24, 32, 128))
					changeCounter(Trait["Capitalist"], getPercent(getCounter(Trait["Capitalist"]), 24, 32, 128))
					bValid = True
				elif iImprovement == Improvement["Winery"]:
					changeCounter(Trait["Commercial"], getPercent(getCounter(Trait["Commercial"]), 12, 18, 48))
					changeCounter(Trait["Charismatic"], getPercent(getCounter(Trait["Charismatic"]), 12, 18, 48))
					bValid = True
				elif iImprovement == Improvement["Cottage"]:
					changeCounter(Trait["Financial"], getPercent(getCounter(Trait["Financial"]), 12, 12, 64))
					changeCounter(Trait["Commercial"], getPercent(getCounter(Trait["Commercial"]), 12, 12, 64))
					bValid = True
				elif iImprovement == Improvement["Fort"]:
					changeCounter(Trait["Protective"], getPercent(getCounter(Trait["Protective"]), 24, 32, 128))
					bValid = True
				elif iImprovement == Improvement["Forest Preserve"]:
					changeCounter(Trait["Humanitarian"], getPercent(getCounter(Trait["Humanitarian"]), 24, 32, 128))
					bValid = True
					
			if bValid:	
				scriptData["hasHadImprovement"] = True
				pPlot.setScriptData(pickle.dumps(scriptData))
				
# ######################################################
# 	City Events
# ######################################################	
	def createCityData(self, pCity):
		'Creates the initial script data of a city'
		gc = CyGlobalContext() 

		scriptData = {}
		for iRel in xrange(gc.getNumReligionInfos()):
			scriptData[iRel] = False
		
		scriptData["size7"] = False
		scriptData["size12"] = False
		scriptData["size18"] = False
		scriptData["size25"] = False
		
		pCity.setScriptData(pickle.dumps(scriptData))
		
	def onCityGrowth(self, argsList):
		'City Population Growth'
		# self.parent.onCityGrowth(self, argsList)
		pCity 			= argsList[0]
		iPlayer 		= argsList[1]
		
		if self.Tools.isBarbarian(iPlayer): return
		
		gc 				= CyGlobalContext() 
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iPlayer)
		
		iPopulation = pCity.getPopulation()
		if iPopulation < 7: return;
		
		try:
			scriptData = (pickle.loads(pCity.getScriptData()))
		except:
			self.createCityData(pCity)
			scriptData = (pickle.loads(pCity.getScriptData()))
		
		Trait 			= self.Traits
		
		getPercent 		= self.Tools.getPercent
		getCounter 		= pPlayer.getTraitCounter
		
		if iPopulation >= 7 and not scriptData["size7"]:
			pPlayer.changeTraitCounter(Trait["Expansive"], getPercent( getCounter(Trait["Expansive"]), 3, 4, 8 ) )
			scriptData["size7"] = True
			pCity.setScriptData(pickle.dumps(scriptData))
		if iPopulation >= 12 and not scriptData["size12"]:
			pPlayer.changeTraitCounter(Trait["Expansive"], getPercent( getCounter(Trait["Expansive"]), 5, 6, 12 ) )
			scriptData["size12"] = True
			pCity.setScriptData(pickle.dumps(scriptData))
		if iPopulation >= 18 and not scriptData["size18"]:
			pPlayer.changeTraitCounter(Trait["Expansive"], getPercent( getCounter(Trait["Expansive"]), 8, 9, 15 ) )
			scriptData["size18"] = True
			pCity.setScriptData(pickle.dumps(scriptData))
		if iPopulation >= 25 and not scriptData["size25"]:
			pPlayer.changeTraitCounter(Trait["Expansive"], getPercent( getCounter(Trait["Expansive"]), 10, 12, 18 ) )
			scriptData["size25"] = True
			pCity.setScriptData(pickle.dumps(scriptData))


	def onCultureExpansion(self, argsList):
		'City Culture Expansion'
		# self.parent.onCultureExpansion(self, argsList)
		pCity 			= argsList[0]
		iPlayer 		= argsList[1]
		
		if self.Tools.isBarbarian(iPlayer): return
		
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		pPlayer 		= gc.getPlayer(iPlayer)
		
		getCounter 		= pPlayer.getTraitCounter
		game 			= CyGame()
		randNum 		= game.getSorenRandNum
		
		Trait 			= self.Traits
		iCreative		= Trait["Creative"]
		
		iRnd = ((randNum((pCity.getCultureLevel()*5), "Creative Random")) / 2)
		if iRnd <= 1: iRnd = 2;
	
		iRnd = getPercent(getCounter(iCreative), iRnd, iRnd)
		pPlayer.changeTraitCounter(iCreative, iRnd)
			
	def onProjectBuilt(self, argsList):
		'Project Completed'
		# self.parent.onProjectBuilt(self, argsList)
		pCity, iProjectType = argsList
		iPlayer = pCity.getOwner()
		
		if self.Tools.isBarbarian(iPlayer): return
		
		gc 			= CyGlobalContext()
		game 		= CyGame()
		pPlayer 	= gc.getPlayer(iPlayer)
		getPercent 	= self.Tools.getPercent
		
		# TODO: Points from projects?
		
		
	def onBuildingBuilt(self, argsList):
		'Building Completed'
		# self.parent.onBuildingBuilt(self, argsList)
		pCity, iBuildingType = argsList
		iPlayer 		= pCity.getOwner()
		Tool			= self.Tools
		
		if Tool.isBarbarian(iPlayer): return

		gc 				= CyGlobalContext() 
		getPercent 		= Tool.getPercent
		pPlayer 		= gc.getPlayer(iPlayer)
		getCounter 		= pPlayer.getTraitCounter
		changeCounter 	= pPlayer.changeTraitCounter
		pPlot 			= pCity.plot()
		game 			= CyGame()
		iB			 	= gc.getBuildingInfo(iBuildingType).getBuildingClassType()
		
		# Traits
		Trait 			= self.Traits
		# Buildings
		Building 		= self.Buildings
		
		changeCounter( Trait["Productive"], 1 ) 
			
		if 	 iB == Building["Versailles"]:	
			iPercent = getPercent( getCounter( Trait["Organized"] ), 75, 48, 64 ) 
			changeCounter( Trait["Organized"], iPercent )
		
		elif iB == Building["Walls"]: 		
			iPercent = getPercent( getCounter( Trait["Protective"] ), 12, 12, 36 )  
			changeCounter( Trait["Protective"], iPercent )
			
		elif iB == Building["Castle"]:	
			iPercent = getPercent( getCounter( Trait["Protective"] ), 18, 24, 48 ) 
			changeCounter( Trait["Protective"], iPercent )
			
		elif iB == Building["Barracks"]:	
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 12, 12, 36 )  
			changeCounter( Trait["Aggressive"], iPercent )
			
		elif iB == Building["Stable"]: 	
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 12, 12, 36 )  
			changeCounter( Trait["Aggressive"], iPercent )
			iPercent = getPercent( getCounter( Trait["Horselords"] ), 20, 24, 52 ) 
			changeCounter( Trait["Horselords"], iPercent )
			
		elif iB == Building["Bunker"]: 		
			iPercent = getPercent( getCounter( Trait["Protective"] ), 18, 24, 48 ) 
			changeCounter( Trait["Protective"], iPercent )
			
		elif iB == Building["Bomb Shelter"]: 
			iPercent = getPercent( getCounter( Trait["Protective"] ), 18, 24, 48 ) 
			changeCounter( Trait["Protective"], iPercent )
			
		elif iB == Building["Granary"]: 
			iPercent = getPercent( getCounter( Trait["Agricultural"] ), 12, 12, 36 )  
			changeCounter( Trait["Agricultural"], iPercent )
			iPercent = getPercent( getCounter( Trait["Expansive"] ), 12, 12, 36 )  
			changeCounter( Trait["Expansive"], iPercent )
			
		elif iB == Building["Aqueduct"]: 
			iPercent = getPercent( getCounter( Trait["Agricultural"] ), 8, 8, 16)  
			changeCounter( Trait["Agricultural"], iPercent )
			iPercent = getPercent( getCounter( Trait["Expansive"] ), 12, 12, 36 )  
			changeCounter( Trait["Expansive"], iPercent )
			
		elif iB == Building["Hospital"]: 
			iPercent = getPercent( getCounter( Trait["Humanitarian"] ), 12, 24, 32 ) 
			changeCounter( Trait["Humanitarian"], iPercent )
			
		elif iB == Building["Recycling Center"]: 
			iPercent = getPercent( getCounter( Trait["Humanitarian"] ), 25, 48, 64 ) 
			changeCounter( Trait["Humanitarian"], iPercent )
		
		elif iB == Building["Lighthouse"]: 
			iPercent = getPercent( getCounter( Trait["Seafaring"] ), 12, 12, 24) 
			changeCounter( Trait["Seafaring"], iPercent )
			
		elif iB == Building["Harbor"]: 
			iPercent = getPercent( getCounter( Trait["Seafaring"] ), 12, 12, 24) 
			changeCounter( Trait["Seafaring"], iPercent )
			iPercent = getPercent( getCounter( Trait["Traders"] ), 12, 12, 36 )  
			changeCounter( Trait["Traders"], iPercent )
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 12, 12, 36 )  
			changeCounter( Trait["Commercial"], iPercent )
			
		elif iB == Building["Custom House"]: 
			iPercent = getPercent( getCounter( Trait["Traders"] ), 22, 24, 48 ) 
			changeCounter( Trait["Traders"], iPercent )
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 6, 12, 24 )
			changeCounter( Trait["Commercial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Financial"] ), 12, 12, 36 )  
			changeCounter( Trait["Financial"], iPercent )
			
		elif iB == Building["Drydock"]: 
			iPercent = getPercent( getCounter( Trait["Seafaring"] ), 12, 12, 36 ) 
			changeCounter( Trait["Seafaring"], iPercent )
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 12, 12, 36 )  
			changeCounter( Trait["Aggressive"], iPercent )
			
		elif iB == Building["Airport"]: 
			changeCounter( Trait["Commercial"], 5 ) 
			changeCounter( Trait["Financial"], 3 )  
			changeCounter( Trait["Traders"], 1 ) 
			iPercent = getPercent( getCounter( Trait["Traders"] ), 12, 12, 36 ) 
			changeCounter( Trait["Traders"], iPercent )
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 6, 12, 24 )
			changeCounter( Trait["Commercial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Financial"] ), 6, 12, 24 )
			changeCounter( Trait["Financial"], iPercent )
			
		elif iB == Building["Forge"]: 
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 12, 12, 36 )  
			changeCounter( Trait["Industrious"], iPercent )
		
		elif iB == Building["Factory"]: 
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 6, 12, 24 )
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Coal Plant"]: 
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 6, 12, 24 )
			changeCounter( Trait["Industrious"], iPercent )
			iPercent = getPercent( getCounter( Trait["Humanitarian"] ), 6, 12, 24 )
			changeCounter( Trait["Humanitarian"], -iPercent )
			
		elif iB == Building["Hydro Plant"]: 
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 6, 12, 24 )
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Nuclear Plant"]:	
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 6, 12, 24 )
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Industrial Park"]:
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 8, 12, 24 ) 
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Obelisk"]:  
			iPercent = getPercent( getCounter( Trait["Creative"] ), 12, 12, 36 )  
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Charismatic"] ), 18, 24, 48 ) 
			changeCounter( Trait["Charismatic"], iPercent )
			
		elif iB == Building["Public Transportation"]: 
			iPercent = getPercent( getCounter( Trait["Humanitarian"] ), 18, 24, 48 ) 
			changeCounter( Trait["Humanitarian"], iPercent )
			
		elif iB == Building["Academy"]: 
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 32, 32, 64 ) 
			changeCounter( Trait["Scientific"], iPercent )
			
		elif iB == Building["Library"]: 
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 12, 12, 36 )  
			changeCounter( Trait["Scientific"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Philosophical"] ), 12, 12, 36 )  
			changeCounter( Trait["Philosophical"], iPercent ) 
			
		elif iB == Building["Observatory"]: 
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 18, 24, 48 )  
			changeCounter( Trait["Scientific"], iPercent ) 
			
		elif iB == Building["Laboratory"]: 
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 32, 32, 64 )   
			changeCounter( Trait["Scientific"], iPercent ) 
			
		elif iB == Building["Theathre"]: 
			iPercent = getPercent( getCounter( Trait["Creative"] ), 18, 24, 48 ) 
			changeCounter( Trait["Creative"], iPercent ) 
			
		elif iB == Building["Colosseum"]: 
			iPercent = getPercent( getCounter( Trait["Creative"] ), 18, 24, 48 ) 
			changeCounter( Trait["Creative"], iPercent ) 
			
		elif iB == Building["Broadcast Tower"]: 
			iPercent = getPercent( getCounter( Trait["Creative"] ), 32, 32, 64 ) 
			changeCounter( Trait["Creative"], iPercent ) 
			
		elif iB == Building["Market"]: 
			iPercent = getPercent( getCounter( Trait["Financial"] ), 12, 18, 36 )
			changeCounter( Trait["Financial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 8, 12, 24 )
			changeCounter( Trait["Commercial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 6, 8, 16 )
			changeCounter( Trait["Capitalist"], iPercent ) 
			
		elif iB == Building["Grocer"]: 
			iPercent = getPercent( getCounter( Trait["Financial"] ), 8, 12, 24 )
			changeCounter( Trait["Financial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 12, 18, 36 )
			changeCounter( Trait["Commercial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 6, 8, 16 )
			changeCounter( Trait["Capitalist"], iPercent ) 
			
		elif iB == Building["Bank"]: 
			iPercent = getPercent( getCounter( Trait["Financial"] ), 5, 8, 16 )
			changeCounter( Trait["Financial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 12, 24, 48 )
			changeCounter( Trait["Commercial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 6, 16, 24 )
			changeCounter( Trait["Capitalist"], iPercent ) 
			
		elif iB == Building["Supermarket"]: 
			iPercent = getPercent( getCounter( Trait["Financial"] ), 12, 18, 36 )
			changeCounter( Trait["Financial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 25, 36, 96 )
			changeCounter( Trait["Commercial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 16, 24, 48 )
			changeCounter( Trait["Capitalist"], iPercent ) 
			
		elif iB == Building["Courthouse"]:
			iPercent = getPercent( getCounter( Trait["Organized"] ), 12, 18, 36 )
			changeCounter( Trait["Organized"], iPercent )
			
		elif iB == Building["Jail"]:
			iPercent = getPercent( getCounter( Trait["Organized"] ), 12, 18, 36 )
			changeCounter( Trait["Organized"], iPercent )
			iPercent = getPercent( getCounter( Trait["Analytical"] ), 16, 24, 48 )
			changeCounter( Trait["Analytical"], iPercent )
			
		elif iB == Building["Levee"]:
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 25, 52, 96 )
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Intelligence Agency"]:
			iPercent = getPercent( getCounter( Trait["Analytical"] ), 25, 36, 96 )
			changeCounter( Trait["Analytical"], iPercent )
			iPercent = getPercent( getCounter( Trait["Nationalist"] ), 4, 4, 12 )  
			changeCounter( Trait["Nationalist"], -iPercent )
			
		elif iB == Building["National Security"]:
			iPercent = getPercent( getCounter( Trait["Analytical"] ), 25, 36, 96 )
			changeCounter( Trait["Analytical"], iPercent )
			iPercent 		= getPercent( getCounter( Trait["Nationalist"] ), 4, 4, 12 )  
			changeCounter( Trait["Nationalist"], -iPercent )
			
		elif Tool.isTemple(iB):
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 4, 4, 12 ) 
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif Tool.isMonastery(iB):
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 5, 6, 16 )
			changeCounter( Trait["Spiritual"], iPercent )
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 3, 4, 8 ) 
			changeCounter( Trait["Scientific"], iPercent ) 
			
		elif Tool.isCathedral(iB):
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 25, 24, 36 )
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif Tool.isShrine(iB):
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 50, 36, 96 )
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Heroic Epic"]:
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 8, 10, 16 ) 
			changeCounter( Trait["Aggressive"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Imperialist"] ), 12, 12, 36 )
			changeCounter( Trait["Imperialist"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Charismatic"] ), 16, 48, 64 )
			changeCounter( Trait["Charismatic"], iPercent ) 
			
		elif iB == Building["National Epic"]:
			iPercent = getPercent( getCounter( Trait["Charismatic"] ), 12, 12, 36 )  
			changeCounter( Trait["Charismatic"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Philosophical"] ), 16, 48, 64 ) 
			changeCounter( Trait["Philosophical"], iPercent ) 
			
		elif iB == Building["Hermitage"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 16, 48, 64 ) 
			changeCounter( Trait["Creative"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Philosophical"] ), 12, 12, 36 )  
			changeCounter( Trait["Philosophical"], iPercent ) 
			
		elif iB == Building["Oxford University"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 12, 12, 36 )  
			changeCounter( Trait["Creative"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Philosophical"] ), 12, 12, 36 )  
			changeCounter( Trait["Philosophical"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 34, 64, 128 )  
			changeCounter( Trait["Scientific"], iPercent ) 
			
		elif iB == Building["Wall Street"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 34, 64, 128 ) 
			changeCounter( Trait["Capitalist"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Financial"] ), 18, 24, 48 )  
			changeCounter( Trait["Financial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 12, 12, 36 )  
			changeCounter( Trait["Commercial"], iPercent ) 
			
		elif iB == Building["Iron Works"]:
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 8, 16, 32 ) 
			changeCounter( Trait["Industrious"], iPercent )
			iPercent = getPercent( getCounter( Trait["Productive"] ), 8, 16, 32 ) 
			changeCounter( Trait["Productive"], iPercent )
			
		elif iB == Building["West Point"]:
			iPercent = getPercent( getCounter( Trait["Charismatic"] ), 34, 64, 128 )   
			changeCounter( Trait["Charismatic"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 18, 24, 48 )  
			changeCounter( Trait["Aggressive"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Imperialist"] ), 12, 12, 36 ) 
			changeCounter( Trait["Imperialist"], iPercent ) 
			
		elif iB == Building["Mt.Rushmore"]:
			iPercent = getPercent( getCounter( Trait["Imperialist"] ), 12, 12, 36 ) 
			changeCounter( Trait["Imperialist"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Organized"] ), 12, 12, 36 ) 
			changeCounter( Trait["Organized"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Creative"] ), 18, 24, 48 )   
			changeCounter( Trait["Creative"], iPercent ) 
			
		elif iB == Building["Red Cross"]:
			iPercent = getPercent( getCounter( Trait["Humanitarian"] ), 34, 64, 128 ) 
			changeCounter( Trait["Humanitarian"], iPercent ) 
			
		elif iB == Building["Scotland Yard"]:
			iPercent = getPercent( getCounter( Trait["Analytical"] ), 25, 52, 96 )  
			changeCounter( Trait["Analytical"], iPercent )
			
		elif iB == Building["Pyramid"]:
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 24, 32, 64 )  
			changeCounter( Trait["Industrious"], iPercent )
			iPercent = getPercent( getCounter( Trait["Democratic"] ), 8, 16, 32 )  
			changeCounter( Trait["Democratic"], iPercent )
			
		elif iB == Building["Stonehenge"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 8, 16, 32 )  
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 8, 16, 32 )  
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Great Library"]:
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 16, 48, 64 )  
			changeCounter( Trait["Scientific"], iPercent )
			iPercent = getPercent( getCounter( Trait["Creative"] ), 8, 8, 16 )  
			changeCounter( Trait["Creative"], iPercent )
			
		elif iB == Building["Great Lighthouse"]:
			iPercent = getPercent( getCounter( Trait["Seafaring"] ), 16, 48, 64 )   
			changeCounter( Trait["Seafaring"], iPercent )
			iPercent = getPercent( getCounter( Trait["Fishermen"] ), 8, 18, 32 )    
			changeCounter( Trait["Fishermen"], iPercent )
			
		elif iB == Building["Hanging Garden"]:
			iPercent = getPercent( getCounter( Trait["Expansive"] ), 8, 16, 32 )    
			changeCounter( Trait["Expansive"], iPercent )
			iPercent = getPercent( getCounter( Trait["Creative"] ), 20, 24, 52 )  
			changeCounter( Trait["Creative"], iPercent )
			
		elif iB == Building["Colossus"]:
			iPercent = getPercent( getCounter( Trait["Seafaring"] ), 8, 16, 32 )   
			changeCounter( Trait["Seafaring"], iPercent )
			iPercent = getPercent( getCounter( Trait["Financial"] ), 6, 12, 24 )
			changeCounter( Trait["Financial"], iPercent ) 
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 5, 8, 16 )
			changeCounter( Trait["Commercial"], iPercent ) 
			
		elif iB == Building["Oracle"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 20, 24, 52 )  
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 20, 24, 52 )  
			changeCounter( Trait["Scientific"], iPercent )
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 20, 24, 52 )  
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Parthenon"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ),  6, 12, 24 )  
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Philosophical"] ),  6, 12, 24 ) 
			changeCounter( Trait["Philosophical"], iPercent )
			
		elif iB == Building["Angkor Wat"]:
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 8, 16, 32 )   
			changeCounter( Trait["Spiritual"], iPercent )
			iPercent = getPercent( getCounter( Trait["Productive"] ), 20, 24, 52 ) 
			changeCounter( Trait["Productive"], iPercent )
			
		elif iB == Building["Hagia Sophia"]:
			iPercent = getPercent( getCounter( Trait["Productive"] ), 8, 16, 32 ) 
			changeCounter( Trait["Productive"], iPercent )
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 20, 24, 52 ) 
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Chicken Itza"]:
			iPercent = getPercent( getCounter( Trait["Organized"] ),  6, 12, 24 )  
			changeCounter( Trait["Organized"], iPercent )
			iPercent = getPercent( getCounter( Trait["Protective"] ),  6, 12, 24 )  
			changeCounter( Trait["Protective"], iPercent )
			
		elif iB == Building["Sistine Chapel"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 9, 18, 36 ) 
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 8, 12, 24 )  
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Notre Dame"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 6, 12, 24 )
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Charismatic"] ), 6, 12, 24 )  
			changeCounter( Trait["Charismatic"], iPercent )
		
		elif iB == Building["Spiral Minaret"]:
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 6, 12, 24 )
			changeCounter( Trait["Spiritual"], iPercent )
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 6, 12, 24 )  
			changeCounter( Trait["Commercial"], iPercent )
		
		elif iB == Building["Taj Mahal"]:
			iPercent = getPercent( getCounter( Trait["Charismatic"] ), 6, 12, 24 )
			changeCounter( Trait["Charismatic"], iPercent )
			iPercent = getPercent( getCounter( Trait["Creative"] ), 6, 12, 24 )  
			changeCounter( Trait["Creative"], iPercent )
			
		elif iB == Building["Kremlin"]:
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 8, 12, 24 ) 
			changeCounter( Trait["Commercial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Analytical"] ), 9, 18, 36 )   
			changeCounter( Trait["Analytical"], iPercent )
			
		elif iB == Building["Eiffel Tower"]:
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 8, 12, 24 ) 
			changeCounter( Trait["Commercial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Creative"] ), 9, 18, 36 )   
			changeCounter( Trait["Creative"], iPercent )
			
		elif iB == Building["Statue of Liberty"]:
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 9, 18, 36 ) 
			changeCounter( Trait["Commercial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Philosophical"] ), 8, 12, 24 ) 
			changeCounter( Trait["Philosophical"], iPercent )
			
		elif iB == Building["Broadway"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 12, 24, 48 )   
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Peaceful"] ),  6, 12, 24 )  
			changeCounter( Trait["Peaceful"], iPercent )
			
		elif iB == Building["Rock'n'Roll"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 12, 24, 48 )   
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Nonviolent"] ),  6, 12, 24 )  
			changeCounter( Trait["Nonviolent"], iPercent )
			
		elif iB == Building["Hollywood"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 12, 24, 48 )    
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Libertarian"] ), 6, 12, 24 ) 
			changeCounter( Trait["Libertarian"], iPercent )
			
		elif iB == Building["Great Dam"]:
			iPercent = getPercent( getCounter( Trait["Productive"] ), 12, 24, 48 )    
			changeCounter( Trait["Productive"], iPercent )
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 12, 24, 48 )    
			changeCounter( Trait["Industrious"], iPercent )
			
		elif iB == Building["Pentagon"]:
			iPercent = getPercent( getCounter( Trait["Imperialist"] ), 34, 64, 128 )     
			changeCounter( Trait["Imperialist"], iPercent )
			iPercent = getPercent( getCounter( Trait["Protective"] ), 6, 12, 24 )    
			changeCounter( Trait["Protective"], iPercent )
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 6, 12, 24 )    
			changeCounter( Trait["Aggressive"], iPercent )
			
		elif iB == Building["United Nations"]:
			iPercent = getPercent( getCounter( Trait["Diplomatic"] ), 50, 128, 256 )    
			changeCounter( Trait["Diplomatic"], iPercent )
			
		elif iB == Building["Space Elevator"]:
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 50, 128, 256 )    
			changeCounter( Trait["Scientific"], iPercent )
			
		elif iB == Building["Military Academy"]:
			iPercent = getPercent( getCounter( Trait["Aggressive"] ), 25, 32, 64 )    
			changeCounter( Trait["Aggressive"], iPercent )
			iPercent = getPercent( getCounter( Trait["Imperialist"] ), 25, 32, 64 )    
			changeCounter( Trait["Imperialist"], iPercent )
			
		elif iB == Building["Artemis"]:
			iPercent = getPercent( getCounter( Trait["Commercial"] ), 8, 16, 32 )    
			changeCounter( Trait["Commercial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Financial"] ), 5, 8, 16 )    
			changeCounter( Trait["Financial"], iPercent )
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 6, 12, 24 )    
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Sankore"]:
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 6, 12, 24 )    
			changeCounter( Trait["Scientific"], iPercent )
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 6, 12, 24 )    
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Great Wall"]:
			iPercent = getPercent( getCounter( Trait["Protective"] ), 25, 32, 64 )     
			changeCounter( Trait["Protective"], iPercent )
			iPercent = getPercent( getCounter( Trait["Analytical"] ), 6, 12, 24 )    
			changeCounter( Trait["Analytical"], iPercent )
			
		elif iB == Building["Statue of Zeus"]:
			iPercent = getPercent( getCounter( Trait["Peaceful"] ), 8, 16, 32 )    
			changeCounter( Trait["Peaceful"], iPercent )
			iPercent = getPercent( getCounter( Trait["Creative"] ), 6, 12, 24 )    
			changeCounter( Trait["Creative"], iPercent )
			
		elif iB == Building["Mausoleum of Maussollos"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), 6, 12, 24 )    
			changeCounter( Trait["Creative"], iPercent )
			iPercent = getPercent( getCounter( Trait["Philosophical"] ), 6, 12, 24 )    
			changeCounter( Trait["Philosophical"], iPercent )
			iPercent = getPercent( getCounter( Trait["Scientific"] ), 12, 12, 36 )     
			changeCounter( Trait["Scientific"], iPercent )
			iPercent = getPercent( getCounter( Trait["Productive"] ), 12, 12, 36 )     
			changeCounter( Trait["Productive"], iPercent )
			
		elif iB == Building["Cristo Redentor"]:
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 50, 128, 256 )    
			changeCounter( Trait["Spiritual"], iPercent )
			
		elif iB == Building["Shwedagon Paya"]:
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 8, 16, 32 )    
			changeCounter( Trait["Spiritual"], iPercent )
			iPercent = getPercent( getCounter( Trait["Creative"] ), 5, 8, 16 )    
			changeCounter( Trait["Creative"], iPercent )
			
		elif iB == Building["Moai Statues"]:
			iPercent = getPercent( getCounter( Trait["Productive"] ), 8, 16, 32 )    
			changeCounter( Trait["Productive"], iPercent )
			iPercent = getPercent( getCounter( Trait["Seafaring"] ), 8, 8, 16 )    
			changeCounter( Trait["Seafaring"], iPercent )
			iPercent = getPercent( getCounter( Trait["Fishermen"] ), 8, 8, 16 )    
			changeCounter( Trait["Fishermen"], iPercent )
			
		elif iB == Building["Corporation1"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Corporation2"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Corporation3"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Corporation4"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Corporation5"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Corporation6"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Corporation7"]:
			iPercent = getPercent( getCounter( Trait["Capitalist"] ), 50, 128, 256 )    
			changeCounter( Trait["Capitalist"], iPercent )
			
		elif iB == Building["Apolistic Palace"]:
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), 8, 16, 32 )    
			changeCounter( Trait["Spiritual"], iPercent )
		
		# General Extra for Wonders to Industrious	
		if gc.getBuildingClassInfo( iB ).getMaxPlayerInstances(  )  > 0:
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 4, 6, 128)
			changeCounter( Trait["Industrious"], iPercent ) 
		elif isWorldWonderClass( iB ) :
			iPercent = getPercent( getCounter( Trait["Industrious"] ), 6, 12, 256)
			changeCounter( Trait["Industrious"], iPercent ) 
		
		
	def onUnitBuilt(self, argsList):
		'Unit Completed'
		# self.parent.onUnitBuilt(self, argsList)
		pCity 	= argsList[0]
		pUnit 	= argsList[1]
		iPlayer = pUnit.getOwner()
		
		if self.Tools.isBarbarian(iPlayer): return

		gc 				  = CyGlobalContext() 
		getPercent 		  = self.Tools.getPercent
		pPlayer 		  = gc.getPlayer(iPlayer)
		
		getCounter 		  = pPlayer.getTraitCounter
		changeCounter 	  = pPlayer.changeTraitCounter
		iUnitCombat 	  = pUnit.getUnitCombatType()
		#if self.UnitCombats == {}:
		#	self.UnitCombats = getUnitCombatDict()
		UnitCombat 		  = self.UnitCombats
		#if self.Traits == {}:
		#	self.Traits   = getTraitDict()
		Trait 			  = self.Traits

		iAggressive 	  = Trait["Aggressive"]
		iExplorers 		  = Trait["Explorers"]
		iImperialist 	  = Trait["Imperialist"]
		iHorselords 	  = Trait["Horselords"]
		iProtective 	  = Trait["Protective"]
		iSeafaring 	  	  = Trait["Seafaring"]
		
		if iUnitCombat   == UnitCombat["Recon"]:
			iPercent 	  = getPercent( getCounter( iExplorers ), 3, 7, 34 )    
			changeCounter(  iExplorers, iPercent )
		elif iUnitCombat == UnitCombat["Archer"]:
			iPercent 	  = getPercent( getCounter( iProtective ), 3, 6, 25 )    
			changeCounter(  iProtective, iPercent )
		elif iUnitCombat == UnitCombat["Mounted"]:
			iPercent 	  = getPercent( getCounter( iHorselords ), 6, 9, 64 )    
			changeCounter(  iHorselords, iPercent )
		elif iUnitCombat == UnitCombat["Melee"]:
			iPercent 	  = getPercent( getCounter( iAggressive ), 6, 9, 34 )    
			changeCounter(  iAggressive, iPercent )
		elif iUnitCombat == UnitCombat["Siege"]:
			iPercent 	  = getPercent( getCounter( iAggressive ), 3, 6, 34 )    
			changeCounter(  iAggressive, iPercent )
			iPercent 	  = getPercent( getCounter( iImperialist ), 4, 9, 34 )    
			changeCounter(  iImperialist, iPercent )
		elif iUnitCombat == UnitCombat["Gun"]:
			iPercent 	  = getPercent( getCounter( iProtective ), 8, 9, 34 )    
			changeCounter(  iProtective, iPercent )
		elif iUnitCombat == UnitCombat["Armor"]:
			iPercent 	  = getPercent( getCounter( iAggressive ), 12, 12, 64 )    
			changeCounter(  iAggressive, iPercent )
		elif iUnitCombat == UnitCombat["Helicopter"]:
			iPercent 	  = getPercent( getCounter( iProtective ), 6, 6, 50 )    
			changeCounter(  iProtective, iPercent )
			iPercent 	  = getPercent( getCounter( iAggressive ), 8, 9, 50 )    
			changeCounter(  iAggressive, iPercent )
		elif iUnitCombat == UnitCombat["Naval"]:
			iPercent 	  = getPercent( getCounter( iSeafaring ), 4, 9, 50 )    
			changeCounter(  iSeafaring, iPercent )
		elif iUnitCombat == UnitCombat["Air"]:
			iPercent 	  = getPercent( getCounter( iProtective ), 8, 12, 50 )    
			changeCounter(  iProtective, iPercent )
			iPercent 	  = getPercent( getCounter( iAggressive ), 8, 12, 50 )    
			changeCounter(  iAggressive, iPercent )
		
	def onGreatPersonBorn(self, argsList):
		'Unit Promoted'
		# self.parent.onGreatPersonBorn(self, argsList)
		pUnit, iPlayer, pCity = argsList
		
		if self.Tools.isBarbarian(iPlayer): return

		gc 				= CyGlobalContext()
		getPercent 		= self.Tools.getPercent
		pPlayer 		= gc.getPlayer(iPlayer)
		Trait 			= self.Traits
		Great 			= self.GreatPeople
		iUnitClass 		= pUnit.getUnitClassType()
		getCounter 		= pPlayer.getTraitCounter
		changeCounter 	= pPlayer.changeTraitCounter
		
		game = CyGame()
		randNum = game.getSorenRandNum
		iRnd = ((randNum(33, "Philosophical Random")) / 2) + 11
		
		iPhilosophical 	= Trait["Philosophical"]
		iPercent = getPercent( getCounter( iPhilosophical ), (iRnd/2), iRnd, (iRnd * 3) )  
		changeCounter(iPhilosophical, iPercent)
		
		if iUnitClass 	== Great["Prophet"]:
			iPercent = getPercent( getCounter( Trait["Spiritual"] ), (iRnd/2), iRnd, (iRnd * 3) )  
			changeCounter(Trait["Spiritual"], iPercent)
		elif iUnitClass == Great["Scientist"]:
			iPercent = getPercent( getCounter( Trait["Scientific"] ), (iRnd/2), iRnd, (iRnd * 3) )    
			changeCounter(Trait["Scientific"], iPercent)
		elif iUnitClass == Great["Merchant"]:
			iPercent = getPercent( getCounter( Trait["Commercial"] ), (iRnd/4), (iRnd/2), ((iRnd/2) * 3) )  
			changeCounter(Trait["Commercial"], iPercent)
			iPercent = getPercent( getCounter( Trait["Financial"] ), (iRnd/4), (iRnd/2), ((iRnd/2) * 3) )  
			changeCounter(Trait["Financial"], iPercent)
		elif iUnitClass == Great["Engineer"]:
			iPercent = getPercent( getCounter( Trait["Industrious"] ), (iRnd/2), iRnd, (iRnd * 3) )  
			changeCounter(Trait["Industrious"], iPercent)
		elif iUnitClass == Great["Artist"]:
			iPercent = getPercent( getCounter( Trait["Creative"] ), (iRnd/2), iRnd, (iRnd * 3) )  
			changeCounter(Trait["Creative"], iPercent)
		elif iUnitClass == Great["General"]:
			iPercent = getPercent( getCounter( Trait["Imperialist"] ), (iRnd/2), iRnd, (iRnd * 3) )    
			changeCounter(Trait["Imperialist"], iPercent)
		elif iUnitClass == Great["Spy"]:
			iPercent = getPercent( getCounter( Trait["Analytical"] ), (iRnd/2), iRnd, (iRnd * 3) )   
			changeCounter(Trait["Analytical"], iPercent)
		

# ######################################################
# 	Unit Events
# ######################################################	
	def onUnitKilled(self, argsList):
		'Unit Killed'
		# self.parent.onUnitKilled(self, argsList)
		return
		pUnit, iAttacker = argsList
		iDefender 		 = pUnit.getOwner()
		gc 				 = CyGlobalContext() 
		getPercent 		 = self.Tools.getPercent
		getPlayer 		 = gc.getPlayer
		pPlayer 		 = getPlayer(iDefender)
		pAttacker 		 = getPlayer(iAttacker)
		
		# Maybe special points instead?
		if self.Tools.isBarbarian(iAttacker): return
		if self.Tools.isBarbarian(iDefender): return

		Trait 			 = self.Traits
		
		getCounter 		 = pAttacker.getTraitCounter
		changeCounter 	 = pAttacker.changeTraitCounter
		
		iAggressive   	 = Trait["Aggressive"]
		iPercent 	  	 = getPercent( getCounter( iAggressive ), 2, 1, 10 )    
		changeCounter( iAggressive, iPercent )
		
		iNonViolent		= Trait["Nonviolent"]
		iPercent 		= getPercent( getCounter( iNonViolent ), 2, 1, 10 )  
		changeCounter( iNonViolent, -iPercent )
		
		iBenevolent		= Trait["Benevolent"]
		iPercent 		= getPercent( getCounter( iBenevolent ), 2, 1, 10 )  
		changeCounter( iBenevolent, -iPercent )
		
		# TRAIT_RAIDERS
		
	def onUnitSpreadReligionAttempt(self, argsList):
		'Unit tries to spread religion to a city'
		# self.parent.onUnitSpreadReligionAttempt(self, argsList)
		pUnit, iReligion, bSuccess = argsList
		iOwner = pUnit.getOwner()
		
		iX = pUnit.getX()
		iY = pUnit.getY()
		pPlot = CyMap().plot(iX, iY)
		pCity = pPlot.getPlotCity()
		
		if bSuccess:
			gc 			= CyGlobalContext() 
			getPercent 	= self.Tools.getPercent
			pPlayer 	= gc.getPlayer(iOwner)
			if self.Traits == {}:
				self.Traits   = getTraitDict()
			Trait 			  = self.Traits
			iSpiritual 	= Trait["Spiritual"]
			iPercent 	= getPercent( pPlayer.getTraitCounter( iSpiritual ), 2, 1, 10 )  
			pPlayer.changeTraitCounter( iSpiritual, iPercent )

			
	def onUnitPillage(self, argsList):
		'Unit pillages a plot'
		# self.parent.onUnitPillage(self, argsList)
		pUnit, iImprovement, iRoute, iOwner = argsList
		gc 				= CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		iPlotX 			= pUnit.getX()
		iPlotY 			= pUnit.getY()
		pPlot 			= CyMap().plot(iPlotX, iPlotY)
		pPlayer 		= gc.getPlayer(pUnit.getOwner())
		
		if pPlayer.isBarbarian(): return
		
		getCounter 		 = pPlayer.getTraitCounter
		changeCounter 	 = pPlayer.changeTraitCounter
		
		Trait 			= self.Traits
		iAggressive	= Trait["Aggressive"]
		iPercent	= getPercent( getCounter( iAggressive ), 2, 1, 10 )  
		changeCounter( iAggressive, iPercent )
		iRaiders 	= Trait["Raiders"]
		iPercent 	= getPercent( getCounter( iRaiders ), 4, 2, 20 )  
		changeCounter( iRaiders, iPercent )
		iNonViolent	= Trait["Nonviolent"]
		iPercent 	= getPercent( getCounter( iNonViolent ), 3, 1, 10 )  
		changeCounter( iNonViolent, -iPercent )
		
		
	def onGoodyReceived(self, argsList):
		'Goody received'
		# self.parent.onGoodyReceived(self, argsList)
		iPlayer, pPlot, pUnit, iGoodyType = argsList
		gc = CyGlobalContext() 
		getPercent 		= self.Tools.getPercent
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iPlayer)
		
		if pPlayer.isBarbarian(): return
		
		iMaxPlayer 		= gc.getMAX_PLAYERS()
		iNumPlayers 	= self.Tools.getNumPlayers()
		
		iValue = iNumPlayers * 2
		if iValue > 20:
			iValue = 20
		elif iValue < 10:
			iValue = 10
		
		Trait 			= self.Traits
		iExplorers		= Trait["Explorers"]
		
		iPercent = getPercent( pPlayer.getTraitCounter( iExplorers ), 25, iValue, (iValue * 3) )  
		pPlayer.changeTraitCounter( iExplorers, iPercent )
	
	
	
	
	
	
	
	
		